import { describe, it, expect, beforeAll, afterAll, beforeEach, afterEach } from '@jest/globals'
import mockfs from 'mock-fs'
import request from 'supertest'
import { createApp } from '../src/createApp'
import { createMockServer, generateSimpleViteManifest, getRandomPort } from './util.js'

describe('Manifest service', () => {
  let app
  let mockserver, mockserver2
  const port = getRandomPort()
  const port2 = getRandomPort()

  beforeAll(() => {
    mockfs({
      './config': {
        'config.yaml': `baseUrls:
          - http://localhost:${port}`
      }
    })
    app = createApp()
  })

  afterAll(() => {
    mockfs.restore()
  })

  beforeEach(async () => {
    mockserver = await createMockServer({ port })
    mockserver.respondWith({
      '/manifest.json': generateSimpleViteManifest({ 'example.js': 'test' }),
      '/example.js': ''
    })
  })

  afterEach(() => {
    mockserver?.close()
    mockserver2?.close()
    process.env.CACHE_TTL = 30000
  })

  // Some say, this is not necessary to test
  // But failing startups due to code errors in the probes will cause this to not work
  // Therefore, we should keep this
  it('is healthy', async () => {
    const response = await request(app).get('/healthy')
    expect(response.statusCode).toBe(200)
  })

  it('fetches manifest data', async () => {
    const response = await request(app).get('/manifests')
    expect(response.statusCode).toBe(200)
    expect(response.body).toEqual([{ namespace: 'test', path: 'example' }])
  })

  it('caches manifest data', async () => {
    const response = await request(app).get('/manifests')
    expect(response.statusCode).toBe(200)
    expect(response.body).toEqual([{ namespace: 'test', path: 'example' }])

    mockserver.close()
    mockserver = await createMockServer({ port })
    mockserver.respondWith({
      '/manifest.json': generateSimpleViteManifest({ 'example.js': 'other' }),
      '/example.js': ''
    })

    await new Promise(resolve => setTimeout(resolve, 150))

    const response2 = await request(app).get('/manifests')
    expect(response2.statusCode).toBe(200)
    expect(response2.body).toEqual([{ namespace: 'test', path: 'example' }])
  })

  it('refreshes manifest data after caching timeout', async () => {
    process.env.CACHE_TTL = 1
    app = createApp()

    const response = await request(app).get('/manifests')
    expect(response.statusCode).toBe(200)
    expect(response.body).toEqual([{ namespace: 'test', path: 'example' }])

    mockserver.close()
    mockserver = await createMockServer({ port })
    mockserver.respondWith({
      '/manifest.json': generateSimpleViteManifest({ 'example.js': 'other' }),
      '/example.js': ''
    })

    // wait some time
    await new Promise(resolve => setTimeout(resolve, 10))

    const response2 = await request(app).get('/manifests')
    expect(response2.statusCode).toBe(200)
    expect(response2.body).toEqual([{ namespace: 'other', path: 'example' }])
  })

  it('can load multiple configurations', async () => {
    mockfs({
      './config': {
        'config.yaml': `baseUrls:
          - http://localhost:${port}
          - http://localhost:${port2}`
      }
    })

    mockserver.close()
    mockserver = await createMockServer({ port })
    mockserver.respondWith({
      '/manifest.json': generateSimpleViteManifest({ 'example1.js': 'other' }),
      '/example1.js': ''
    })
    mockserver2 = await createMockServer({ port: port2 })
    mockserver2.respondWith({
      '/manifest.json': generateSimpleViteManifest({ 'example2.js': 'thing' }),
      '/example2.js': ''
    })

    process.env.CACHE_TTL = 1
    const app = createApp()

    await request(app)
      .get('/manifests')
      .then(response => {
        expect(response.statusCode).toBe(200)
        expect(response.body).toEqual([
          { namespace: 'other', path: 'example1' },
          { namespace: 'thing', path: 'example2' }
        ])
      })
  })
})
